"""
Module contains settings groups classes
"""

from typing import Dict, List, Literal, Optional, Tuple, Union

from configs_core.base_classes import BaseSettingClass

from .limits_classes import CrossMatchLimits, MatchLimits


class LoggerSetting(BaseSettingClass):
    """
    Logger settings class
    Attributes:
        logLevel(str): log level
        folderWithLogs(str): folder for store logs
        maxLogFileSize(str): maximum log file size (for log rotation)
        logTime(str): log time (utc or local)
        logToStdOut (bool): log to stdout
        logToFile (bool): log to a file
        multilineStackTrace (bool): allow a multiline line stack trace message
    """

    def __init__(self, serviceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName(str): service name
            settings(str): dict with settings
        """

        super().__init__(serviceName, "{}_LOGGER".format(serviceName.replace("-", "_").upper()))
        self.logLevel = settings["log_level"]
        self.folderWithLogs = settings["folder_with_logs"]
        self.maxLogFileSize = settings["max_log_file_size"]
        self.logTime = settings["log_time"]
        self.logToStdOut = settings["log_to_stdout"]
        self.logToFile = settings["log_to_file"]
        self.multilineStackTrace = settings["multiline_stack_trace"]
        self.format = settings["format"]


class DBSettings:
    """
    DB settings
    Attributes:
        connectionPoolSize (int): connection pool size
    """

    __slots__ = ("connectionPoolSize",)

    def __init__(self, connectionPoolSize: str):
        self.connectionPoolSize = connectionPoolSize

    def asDict(self) -> dict:
        """Get db settings as dict"""
        return {"connection_pool_size": self.connectionPoolSize}

    def __eq__(self, other):
        if not isinstance(other, DBSettings):
            return NotImplemented
        return self.connectionPoolSize == other.connectionPoolSize


class DBSetting(BaseSettingClass):
    """
    DB settings class
    Attributes:
        type(str): db type (postgres/oracle)
        dsn(str): db connection URI in the following format: user:pass@host:port/database?option=value
        userName(str): db user name
        password(str): db password
        dbSettings(DBSettings): db extra settings
    """

    def __init__(self, serviceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """
        from luna_db_tools.model import parseConnectArgs  # pylint: disable-msg=import-outside-toplevel

        super().__init__(serviceName, "{}_DB".format(serviceName.replace("-", "_").upper()))
        self.type = settings["db_type"]
        self.dsn, self.userName, self.password = parseConnectArgs(
            dsn=settings.get("dsn", ""),
            host=settings["db_host"],
            port=settings["db_port"],
            dbname=settings["db_name"],
            user=settings["db_user"],
            password=settings["db_password"],
        )
        self.dbSettings = DBSettings(settings["db_settings"]["connection_pool_size"])

        if serviceName == "luna-events" and self.type == "oracle":
            raise ValueError("Unsupported database type")

    @property
    def dbUrl(self) -> str:
        """
        DB url for sqlalchemy(driver+host+port+user_name+password)
        Returns:
            sqlalchemy db url
        """
        _dbDriver = {"postgres": "postgresql", "oracle": "oracle+cx_oracle"}
        return f"{_dbDriver[self.type]}://{self.userName}:{self.password}@{self.dsn}"

    @property
    def alembicDbUrl(self) -> str:
        """
        DB url for alembic(driver+host+port+user_name+password)
        Returns:
            alembic db url (multiple hosts not supported)
        """
        from luna_db_tools.model import parseConnectArgs  # pylint: disable-msg=import-outside-toplevel

        _dbDriver = {"postgres": "postgresql", "oracle": "oracle+cx_oracle"}
        dsn, _, _ = parseConnectArgs(self.dsn, None, None, None, None, None, enableMultihost=False)
        return f"{_dbDriver[self.type]}://{self.userName}:{self.password}@{dsn}"


class S3Settings(BaseSettingClass):
    """
    S3 storage settings
    Attributes:
        host(str): s3 host
        region(str): s3 region
        awsPublicAccessKey(str): aws public key
        awsSecretAccessKey(str): aws secret key
        requestTimeout(int): aws request timeout
    """

    def __init__(self, serviceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, "S3")
        self.host = settings["host"]
        self.region = settings["region"]
        self.awsPublicAccessKey = settings["aws_public_access_key"]
        self.awsSecretAccessKey = settings["aws_secret_access_key"]
        self.authorizationSignature = settings["authorization_signature"]
        self.requestTimeout = settings["request_timeout"]
        self.verifySSL = settings["verify_ssl"]


class Authorization(BaseSettingClass):
    """
    Configurator auth settings.
    Attributes:
        use_authorization(bool): use auth.
        user(str): login.
        password(str): password.
    """

    def __init__(self, serviceName: str, settings: dict):
        super().__init__(serviceName, "LUNA_CONFIGURATOR_AUTHORIZATION")
        self.useAuthorization = settings["use_authorization"]
        self.user = settings["luna_configurator_user"]
        self.password = settings["luna_configurator_pass"]


class InfluxCredentials:
    """
    Container for influx 2.x credentials
    Attributes:
        organization: organization
        token: authentication token
        bucket: bucket name
    """

    __slots__ = (
        "organization",
        "token",
        "bucket",
    )

    def __init__(self, organization: str, token: str, bucket: str):
        self.organization = organization
        self.token = token
        self.bucket = bucket

    def asDict(self) -> dict:
        """Get influx settings as dict"""
        return {"organization": self.organization, "token": self.token, "bucket": self.bucket}


class MonitoringSettings(BaseSettingClass):
    """
    Monitoring settings class
    Attributes:
        sendData(int): send monitoring data flag
        useSsl(int): ssl using flag
        flushingPeriod(int): flushing period
        host(str): monitoring host
        port(int): monitoring port
        credentials(InfluxCredentials): monitoring database credentials
    """

    def __init__(self, serviceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, "LUNA_MONITORING")
        self.storageType = settings["storage_type"]
        if self.storageType == "influx":
            self.sendData = settings["send_data_for_monitoring"]
            self.useSsl = settings["use_ssl"]
            self.flushingPeriod = settings["flushing_period"]
            self.host = settings["host"]
            self.port = settings["port"]
            self.credentials = InfluxCredentials(
                organization=settings["organization"], token=settings["token"], bucket=settings["bucket"]
            )


class MetricsSettings(BaseSettingClass):
    """
    Metrics settings class.
    Attributes:
        enabled(bool): Whether to enable metrics
        metricsFormat(str): metrics response format
        extraLabels(dict): extra metrics labels
    """

    def __init__(self, serviceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: Service name
            settings: dict with settings
        """
        super().__init__(serviceName, "LUNA_SERVICE_METRICS")
        self.enabled = settings["enabled"]
        self.metricsFormat = settings["metrics_format"]
        self.extraLabels = settings["extra_labels"]


class ServiceAddressSettings(BaseSettingClass):
    """
    Service address settings class
    Attributes:
        origin(str): service origin
        apiVersion(int): service api version
    """

    def __init__(self, serviceName: str, requiredServiceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            requiredServiceName: name of required service
            settings: dict with settings
        """

        super().__init__(serviceName, requiredServiceName)
        self.origin = settings["origin"]
        self.apiVersion = settings["api_version"]


class ImageStoreAddressSettings(ServiceAddressSettings):
    """
    Image store address settings class
    Attributes:
        bucket(str): luna-image-store bucket to use
    """

    def __init__(self, serviceName: str, requiredServiceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            requiredServiceName: name of required service
            settings: dict with settings
        """

        super().__init__(serviceName, requiredServiceName, settings)
        self.bucket = settings["bucket"]


class ServiceTimeoutsSettings(BaseSettingClass):
    """
    Service timeouts settings class
    Attributes:
        connectTimeout(float): connect timeout
        totalTimeout(float): request (total) timeout
        sockConnectTimeout(float): sock connect timeout
        sockReadTimeout(float): sock read timeout
    """

    def __init__(self, serviceName: str, requiredServiceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            requiredServiceName: name of required service
            settings: dict with settings
        """

        super().__init__(serviceName, requiredServiceName)
        self.connectTimeout = settings["connect"]
        self.totalTimeout = settings["request"]
        self.sockConnectTimeout = settings["sock_connect"]
        self.sockReadTimeout = settings["sock_read"]


class RedisSentinel:
    """
    Redis sentinel settings
    Attributes:
        masterName (str): master name sentinel
        sentinels (List[Tuple[str, int]]): list sentinel (host + port)
        user (str): sentinel user name
        password (str): sentinel password
    """

    def __init__(self, sentinelJson: Dict[str, Union[List[str], str]]):
        self.masterName: str = sentinelJson["master_name"]
        sentinels = [sentinel.split(":") for sentinel in sentinelJson["sentinels"]]
        self.sentinels: List[Tuple[str, int]] = [(host, int(port)) for host, port in sentinels]
        self.user = sentinelJson["user"]
        self.password = sentinelJson["password"]

    def __eq__(self, other) -> bool:
        """
        Operator equal

        Args:
            other: other settings

        Returns:
            True if dicts equals, otherwise False
        Raises:
            TypeError: if type 'other' is not RedisSentinel
        """
        if isinstance(other, RedisSentinel):
            if (self.user, self.password) != (other.user, other.password):
                return False
            return self.masterName == other.masterName and set(self.sentinels) == set(other.sentinels)
        raise TypeError(f"Got wrong type {type(other)}.")


class RedisDBSetting(BaseSettingClass):
    """
    Redis db address setting
    Attributes:
        host(str): redis host
        port(int): redis port
        user(str): redis username
        password(str): redis password
        channel(str): redis channel
        sentinel: An RedisSentinel instance with configuration for sentinel
    """

    def __init__(self, serviceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, "REDIS_DB_ADDRESS")
        self.host = settings["host"]
        self.port = settings["port"]
        self.user = settings["user"]
        self.password = settings["password"]
        self.channel = settings["channel"]
        self.sentinel = RedisSentinel(settings["sentinel"])


class RedisStorageSetting(BaseSettingClass):
    """
    Redis db address setting
    Attributes:
        host(str): redis host
        port(int): redis port
        user(str): redis username
        password(str): redis password
        dbNumber(int): redis database number
    """

    def __init__(self, serviceName: str, settingName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settingName: current setting name
            settings: dict with settings
        """

        super().__init__(serviceName, settingName)
        self.host = settings["host"]
        self.port = settings["port"]
        self.user = settings["user"]
        self.password = settings["password"]
        self.dbNumber = settings["number"]
        sentinel = settings["sentinel"]

        if sentinel:
            self.sentinel: Optional[RedisSentinel] = RedisSentinel(sentinel)
        else:
            self.sentinel = None


class AdditionalServicesUsageSetting(BaseSettingClass):
    """
    Support services setting

    Attributes:
        lunaSender(bool): whether to use Luna Sender
        lunaEvents(bool): whether to use Luna Events
        lunaTasks(bool): whether to use Luna Tasks
        lunaMatcherProxy(bool): whether to use Luna Python Matcher Proxy (Sharded matching)
        lunaImageStore(bool): whether to use Luna Image store
        lunaLambda(bool): whether to use Luna lambda
        lunaVideoManager(bool): whether to use Luna video manager
        lunaVideoAgent(bool): whether to use Luna video agent
        lunaVideoManager(bool): whether to use Luna video manager
        lunaStreamsRetranslator(bool): whether to use Luna Streams retranslator
    """

    def __init__(self, serviceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, "ADDITIONAL_SERVICES_USAGE")
        self.lunaSender = settings["luna_sender"]
        self.lunaEvents = settings["luna_events"]
        self.lunaTasks = settings["luna_tasks"]
        self.lunaMatcherProxy = settings["luna_matcher_proxy"]
        self.lunaImageStore = settings["luna_image_store"]
        self.lunaHandlers = settings["luna_handlers"]
        self.lunaLambda = settings["luna_lambda"]
        self.lunaVideoManager = settings["luna_video_manager"]
        self.lunaVideoAgent = settings["luna_video_agent"]
        self.lunaStreamsRetranslator = settings["luna_streams_retranslator"]


class FaceDetector3Setting(BaseSettingClass):
    """
    Face detector v3 settings.

    Attributes:
        minFaceSize (int): min face size
        redetectFaceTargetSize (int): target face size for redetect
        redetectTensorSize (int): target face after preprocessing for redetect
        redetectScoreThreshold (float): redetect score threshold
        scoreThreshold (float): body detection score threshold
    """

    def __init__(self, serviceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, "FACE_DETECTOR_V3")
        self.minFaceSize = settings["min_face_size"]
        self.scoreThreshold = settings["score_threshold"]
        self.redetectFaceTargetSize = settings["redetect_face_target_size"]
        self.redetectTensorSize = settings["redetect_tensor_size"]
        self.redetectScoreThreshold = settings["redetect_score_threshold"]


class BodyDetectorSetting(BaseSettingClass):
    """
    Human body detector settings.

    Attributes:
        imageSize (int): image size for down scaling
        scoreThreshold (float): body detection score threshold
        redetectScoreThreshold (float): redetect score threshold

    """

    def __init__(self, serviceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, "BODY_DETECTOR")
        self.imageSize = settings["image_size"]
        self.scoreThreshold = settings["score_threshold"]
        self.redetectScoreThreshold = settings["redetect_score_threshold"]


class RuntimeSetting(BaseSettingClass):
    """
    Runtime settings

    Attributes:
        globalDeviceClass (str): execution device type - cpu, gpu
        numThreads (int): number of worker threads
        numComputeStreams (int): number of streams; Increases performance, but works only with new versions of
                                 NVIDIA driversnumber of streams; Increases performance, but works only with new
                                 versions of NVIDIA drivers
    """

    def __init__(self, serviceName: str, requiredServiceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, requiredServiceName)
        self.globalDeviceClass = settings["global_device_class"]
        self.numThreads = settings["num_threads"]
        self.numComputeStreams = settings["num_compute_streams"]


class AgentSettings(BaseSettingClass):
    """
    Agent extended settings

    Attributes:
        maxActiveStreams (int): number of active streams
    """

    def __init__(self, serviceName: str, requiredServiceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, requiredServiceName)
        self.maxActiveStreams = settings["max_active_streams"]


class EstimatorRuntimeSetting(BaseSettingClass):
    """
    Estimator runtime setting

    Attributes:
        deviceClass (str): execution device type - global, cpu, gpu
        optimalBatchSize (int): optimal batch size for stage
        workerCount (int): stage worker count
    """

    def __init__(self, serviceName: str, requiredServiceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, requiredServiceName)
        self.deviceClass = settings["device_class"]
        self.optimalBatchSize = settings["optimal_batch_size"]
        self.workerCount = settings["worker_count"]


class RedisKeyStorageSetting(BaseSettingClass):
    """
    Redis storage setting

    Attributes:
        defaultTTL (integer): default key ttl
        maxTTL (integer): max key ttl
    """

    def __init__(self, serviceName: str, requiredServiceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, requiredServiceName)
        self.defaultTTL: int = settings["default_ttl"]
        self.maxTTL: int = settings["max_ttl"]


class MatchingCacheSettings:
    """
    Cache matching settings
    Attributes:
        threadCount: thread count for matching, 0 - auto select
        tasksCount: Number of workers that process cached matcher queue
        batchSize: Maximum number of match requests within single batch in cached matcher
    """

    def __init__(self, settings: dict):
        self.threadCount = settings["thread_count"]
        self.tasksCount = settings["tasks_count"]
        self.batchSize = settings["batch_size"]


class RPCSettings:
    """
    RPC client settings

    Attributes:
        timeouts: RPC client timeouts, in milliseconds
        poolSize: Number of connections between lpm and cached matcher processes
    """

    class Timeouts:
        """
        RPC client timeouts

        Attributes:
            connectTimeout: connect timeout, in milliseconds
            requestTimeout: request timeout, in milliseconds
            responseTimeout: response timeout, in milliseconds
        """

        def __init__(self, settings: dict):
            self.connectTimeout = settings["connect_timeout"] * 1000
            self.requestTimeout = settings["request_timeout"] * 1000
            self.responseTimeout = settings["response_timeout"] * 1000

        def asDict(self) -> Dict[str, int]:
            """Returns dict representation"""
            return {key: value for key, value in self.__dict__.items() if not key.startswith("_")}

        def __eq__(self, other):
            if isinstance(other, RPCSettings.Timeouts):
                return self.asDict() == other.asDict()
            raise TypeError(f"Got wrong type {type(other)}.")

    def __init__(self, settings: dict):
        self.timeouts = RPCSettings.Timeouts(settings["timeouts"])
        self.poolSize = settings["pool_size"]

    def __eq__(self, other):
        if isinstance(other, RPCSettings):
            return self.timeouts == other.timeouts
        raise TypeError(f"Got wrong type {type(other)}.")


class CachedLists:
    """
    Caching faces lists settings

    Attributes:
        include: lists which should be caching
        exclude: lists which should not be caching
    """

    def __init__(self, settings: dict):
        self.include: Optional[list[str]] = settings["include"]
        self.exclude: Optional[list[str]] = settings["exclude"]


class CachedData:
    """
    Caching settings

    Attributes:
        facesListsCache: faces lists caching settings
    """

    def __init__(self, settings: dict):
        self.facesListsCache = CachedLists(settings["faces_lists"])


class DescriptorCachingSetting(BaseSettingClass):
    """
    Descriptor caching settings

    Attributes:
        cacheEnabled (integer): use or not cache for matching
        updatingCacheInterval (integer): updating cache interval
        matchingCacheSettings (MatchingCacheSettings): cache matching settings
        rpcSettings (RPCSettings): rpc client settings
    """

    def __init__(self, serviceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, "DESCRIPTORS_CACHE")
        self.cacheEnabled: int = settings["cache_enabled"]
        self.updatingCacheInterval: float = settings["updating_cache_interval"]
        self.matchingCacheSettings = MatchingCacheSettings(settings["matching_settings"])
        self.rpcSettings = RPCSettings(settings["rpc_settings"])
        self.cachedData = CachedData(settings["cached_data"])


class Liveness2Settings(BaseSettingClass):
    """
    Liveness2 settings.

    Attributes:
        realThreshold (float): real threshold
    """

    def __init__(self, serviceName: str, settingGroupName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, settingGroupName)
        self.realThreshold: float = settings["real_threshold"]


class DeepfakeSettings(BaseSettingClass):
    """
    Deepfake settings.

    Attributes:
        realThreshold (float): real threshold
        estimationMode (int): default deepfake estimation mode
    """

    def __init__(self, serviceName: str, settingGroupName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, settingGroupName)
        self.realThreshold: float = settings["real_threshold"]
        self.estimationMode: int = settings["estimation_mode"]


class CommonQualityCheck:
    """Base quality check settings."""

    def __init__(self, settings: dict):
        self.threshold = settings["threshold"]
        for section, value in settings.get("config", {}).items():
            setattr(self, section, RangeQualityCheck(value))


class RangeQualityCheck:
    """Range quality check settings."""

    def __init__(self, settings: dict):
        self.minThreshold = settings["threshold"].get("min")
        self.maxThreshold = settings["threshold"].get("max")

    def asDict(self):
        """Prepare representation."""

        res = {}
        if self.minThreshold is not None:
            res["min"] = self.minThreshold
        if self.maxThreshold is not None:
            res["max"] = self.maxThreshold
        return res


class FaceQualitySettings(BaseSettingClass):
    """Face quality settings."""

    def __init__(self, serviceName: str, settingGroupName: str, settings: dict):
        super().__init__(serviceName, settingGroupName)
        self.imageFormat = CommonQualityCheck(settings["image_format"])
        self.illumination = RangeQualityCheck(settings["illumination"])
        self.specularity = RangeQualityCheck(settings["specularity"])
        self.blurriness = RangeQualityCheck(settings["blurriness"])
        self.dark = RangeQualityCheck(settings["dark"])
        self.light = RangeQualityCheck(settings["light"])
        self.headYaw = RangeQualityCheck(settings["head_yaw"])
        self.headPitch = RangeQualityCheck(settings["head_pitch"])
        self.headRoll = RangeQualityCheck(settings["head_roll"])
        self.gazeYaw = RangeQualityCheck(settings["gaze_yaw"])
        self.gazePitch = RangeQualityCheck(settings["gaze_pitch"])
        self.mouthSmiling = RangeQualityCheck(settings["mouth_smiling"])
        self.mouthOccluded = RangeQualityCheck(settings["mouth_occluded"])
        self.mouthOpen = RangeQualityCheck(settings["mouth_open"])
        self.glasses = CommonQualityCheck(settings["glasses"])
        self.eyes = CommonQualityCheck(settings["eyes"])
        self.headHorizontalCenter = RangeQualityCheck(settings["head_horizontal_center"])
        self.headVerticalCenter = RangeQualityCheck(settings["head_vertical_center"])
        self.headWidth = RangeQualityCheck(settings["head_width"])
        self.headHeight = RangeQualityCheck(settings["head_height"])
        self.eyeDistance = RangeQualityCheck(settings["eye_distance"])
        self.imageWidth = RangeQualityCheck(settings["image_width"])
        self.imageHeight = RangeQualityCheck(settings["image_height"])
        self.aspectRatio = RangeQualityCheck(settings["aspect_ratio"])
        self.faceWidth = RangeQualityCheck(settings["face_width"])
        self.faceHeight = RangeQualityCheck(settings["face_height"])
        self.indentLeft = RangeQualityCheck(settings["indent_left"])
        self.indentRight = RangeQualityCheck(settings["indent_right"])
        self.indentUpper = RangeQualityCheck(settings["indent_upper"])
        self.indentLower = RangeQualityCheck(settings["indent_lower"])
        self.imageSize = RangeQualityCheck(settings["image_size"])
        self.illuminationUniformity = RangeQualityCheck(settings["illumination_uniformity"])
        self.dynamicRange = RangeQualityCheck(settings["dynamic_range"])
        self.eyebrows = CommonQualityCheck(settings["eyebrows"])
        self.shoulders = CommonQualityCheck(settings["shoulders"])
        self.smile = CommonQualityCheck(settings["smile"])
        self.headwear = CommonQualityCheck(settings["headwear"])
        self.naturalLight = CommonQualityCheck(settings["natural_light"])
        self.fishEye = CommonQualityCheck(settings["fish_eye"])
        self.redEye = CommonQualityCheck(settings["red_eye"])
        self.colorType = CommonQualityCheck(settings["color_type"])
        self.backgroundLightness = RangeQualityCheck(settings["background_lightness"])
        self.backgroundUniformity = RangeQualityCheck(settings["background_uniformity"])
        self.faceOcclusion = CommonQualityCheck(settings["face_occlusion"])
        self.foreheadOcclusion = CommonQualityCheck(settings["forehead_occlusion"])
        self.lowerFaceOcclusion = CommonQualityCheck(settings["lower_face_occlusion"])
        self.noseOcclusion = CommonQualityCheck(settings["nose_occlusion"])


class VideoSettings(BaseSettingClass):
    """
    Video processing settings.

    Attributes:
        deviceClass (Literal["cpu", "gpu", "auto"]): decoder device class
        workerCount (int): number of decoder workers
        storage (str): video temporary storage path
        maxSize (int): video max size, in megabytes
        framePoolSize (int): number of pre-allocated frames for video stream
        ffmpegThreadCount (int): number of threads ffmpeg uses when decoding
        timeouts (ServiceTimeoutsSettings): Object with timeout settings
    """

    def __init__(self, serviceName: str, settingGroupName: str, settings: dict):
        super().__init__(serviceName, settingGroupName)
        self.deviceClass: Literal["cpu", "gpu", "auto"] = settings["decoder_device_class"]
        self.workerCount: int = settings["decoder_worker_count"]
        self.storage: str = settings["storage"]
        self.maxSize: int = settings["max_size"]
        self.timeouts: ServiceTimeoutsSettings = ServiceTimeoutsSettings(
            serviceName, settingGroupName, settings["timeouts"]
        )
        self.framePoolSize: int = settings["frame_pool_size"]
        self.ffmpegThreadCount: int = settings["ffmpeg_thread_count"]


class AgentVideoSettings(BaseSettingClass):
    """
    Luna agent video processing settings.

    Attributes:
        deviceClass (Literal["cpu", "gpu", "auto"]): decoder device class
        workerCount (int): number of decoder workers
        maxSize (int): video max size, in megabytes
        timeouts (ServiceTimeoutsSettings): timeouts settings for video saver
        framePoolSize (int): number of pre-allocated frames for video stream
        ffmpegThreadCount (int): number of threads ffmpeg uses when decoding
    """

    def __init__(self, serviceName: str, settingGroupName: str, settings: dict):
        super().__init__(serviceName, settingGroupName)
        self.deviceClass: Literal["cpu", "gpu", "auto"] = settings["decoder_device_class"]
        self.workerCount: int = settings["decoder_worker_count"]
        self.maxSize: int = settings["max_size"]
        self.timeouts: ServiceTimeoutsSettings = ServiceTimeoutsSettings(
            serviceName, settingGroupName, settings["timeouts"]
        )
        self.framePoolSize: int = settings["frame_pool_size"]
        self.ffmpegThreadCount: int = settings["ffmpeg_thread_count"]


class PlatformLimitsSettings(BaseSettingClass):
    """
    Platform limits setting

    Attributes:
        match (MatchLimits): matching limits
        crossMatch (CrossMatchLimits): cross match limits
    """

    def __init__(self, serviceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, "MATCHING_LIMITS")
        self.match = MatchLimits(settings["match"])
        self.crossMatch = CrossMatchLimits(settings["cross_match"])


class RemoteSDKLimitsSettings(BaseSettingClass):
    """
    Luna-handlers limits settings

    Attributes:
        receivedImagesLimit (integer): max number of images received in request
    """

    def __init__(self, serviceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, "LUNA_REMOTE_SDK_LIMITS")
        self.receivedImagesLimit = settings["received_images_limit"]


class HandlersLimitsSettings(BaseSettingClass):
    """
    Luna-remote-sdk limits settings

    Attributes:
        receivedImagesLimit (integer): max number of images received in request
        rawEventDetectionsLimit (integer): max detections in raw event
        rawEventArraysLimit (integer): max length of arrays in raw event
        resultCandidateLimit (integer): result candidate limit
    """

    def __init__(self, serviceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, "LUNA_HANDLERS_LIMITS")
        self.receivedImagesLimit = settings["received_images_limit"]
        self.rawEventDetectionsLimit = settings["raw_event_detections_limit"]
        self.rawEventArraysLimit = settings["raw_event_arrays_limit"]
        self.resultCandidateLimit = settings["result_candidate_limit"]


class ServiceHttpSettings(BaseSettingClass):
    """
    Logger settings class
    Attributes:
        requestTimeout(int): service request timeout
        responseTimeout(int): service response timeout
        requestMaxSize(int): max request size
        keepAliveTimeout(int): keep alive timeout
    """

    def __init__(self, serviceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName(str): service name
            settings(str): dict with settings
        """

        super().__init__(serviceName, "{}_HTTP_SETTINGS".format(serviceName.replace("-", "_").upper()))
        self.requestTimeout = settings["request_timeout"]
        self.responseTimeout = settings["response_timeout"]
        self.requestMaxSize = settings["request_max_size"]
        self.keepAliveTimeout = settings["keep_alive_timeout"]


class VaultSource:
    """Vault key data"""

    def __init__(self, url: str, token: str):
        """
        Initialize
        Args:
            url(str): url for receiving the key
            token(str): authentication token
        """
        self.url = url
        self.token = token

    def asDict(self) -> dict:
        """Returns dict representation"""
        return {"url": self.url, "token": self.token}


class EncryptionParams:
    """Descriptor encryption settings"""

    def __init__(self, source, key: str | dict):
        """
        Initialize
        Args:
            source(str): encryption key source
            key(str | VaultSource): key data
        """
        self.source = source
        self.key = key if source == "raw" else VaultSource(**key)

    def asDict(self) -> dict:
        """Returns dict representation"""
        return {"source": self.source, "key": self.key if isinstance(self.key, str) else self.key.asDict()}


class DescriptorEncryption(BaseSettingClass):
    """
    Descriptor encryption settings
    Attributes:
        enabled: whether encryption enabled or not
        algorithm: encryption algorithm
        params: encryption params
    """

    def __init__(self, serviceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName(str): service name
            settings(str): dict with settings
        """
        super().__init__(serviceName, "LUNA_ENCRYPTION_SETTINGS")
        self.enabled = settings["enabled"]
        if self.enabled:
            self.algorithm = settings["algorithm"]
            self.params = EncryptionParams(**settings["params"])
        else:
            self.algorithm = None
            self.params = None

    def asDict(self) -> dict:
        """Returns dict representation"""
        return {
            "enabled": self.enabled,
            "algorithm": self.algorithm,
            "params": self.params.asDict() if self.params else None,
        }


class TESettings(BaseSettingClass):
    """
    TE settings.

    Attributes:
        detectorStep (int): true detector run rate
        scaleResultSize (int): frame will be scaled to this size in pixels for detection step
          (by the max dimension of width/height)
        skipFrames (int): If track wasn't updated by detect/redetect for this number of frames,
          then track is finished ('18' by default)
    """

    def __init__(self, serviceName: str, settingGroupName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, settingGroupName)
        self.detectorStep: int = settings["detector_step"]
        self.scaleResultSize: int = settings["scale_result_size"]
        self.skipFrames: int = settings["skip_frames"]


class TERuntimeSetting(BaseSettingClass):
    """
    Estimator runtime setting

    Attributes:
        deviceClass (str): execution device type - global, cpu, gpu
        optimalBatchSize (int): optimal batch size for stage
    """

    def __init__(self, serviceName: str, requiredServiceName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settings: dict with settings
        """

        super().__init__(serviceName, requiredServiceName)
        self.deviceClass = settings["device_class"]
        self.optimalBatchSize = settings["optimal_batch_size"]


ServiceAddress = ServiceAddressSettings
ServiceTimeouts = ServiceTimeoutsSettings


class AddressSetting(BaseSettingClass):
    """
    Simple address setting
    Attributes:
        host(str): host
        port(int): port
    """

    def __init__(self, serviceName: str, settingName: str, settings: dict):
        """
        Initialize
        Args:
            serviceName: service name
            settingName: current setting name
            settings: dict with settings
        """

        super().__init__(serviceName, settingName)
        self.host = settings["host"]
        self.port = settings["port"]
