"""
Module contains settings class for luna-lambda-unit
"""

from .configs.configs.services.base_python_service_settings import BasePythonServiceSettingsClass
from .configs.configs.settings.all_settings import SETTINGS
from .configs.configs.settings.classes import (
    AdditionalServicesUsageSetting as _AdditionalServicesUsageSetting,
    ImageStoreAddressSettings as _ImageStoreAddressSettings,
    ServiceAddressSettings as _ServiceAddressSettings,
    ServiceTimeoutsSettings as _ServiceTimeoutsSettings,
)


class ServiceSettings(BasePythonServiceSettingsClass):
    """
    Settings class for luna-lambda-unit
    Attributes:
        faceSamplesStorage(ImageStoreAddressSettings): face samples storage settings
        bodySamplesStorage(ImageStoreAddressSettings): body samples storage settings
        imageOriginStorage(ImageStoreAddressSettings): image origin storage settings
        faceSamplesStorageTimeouts(ServiceTimeoutsSettings): face samples storage timeouts settings
        bodySamplesStorageTimeouts(ServiceTimeoutsSettings): body samples storage timeouts settings
        imageOriginStorageTimeouts(ServiceTimeoutsSettings): image origin storage timeouts settings
        facesAddress(ServiceAddressSettings): luna-faces address settings
        facesTimeouts(ServiceTimeoutsSettings): luna-faces timeouts settings
        eventsAddress(ServiceAddressSettings): luna-events address settings
        eventsTimeouts(ServiceTimeoutsSettings): luna-events timeouts settings
        pythonMatcherAddress(ServiceAddressSettings): luna-python-matcher address settings
        pythonMatcherTimeouts(ServiceTimeoutsSettings): luna-python-matcher timeouts settings
        matcherProxyAddress(ServiceAddressSettings): luna-python-matcher address settings
        matcherProxyTimeouts(ServiceAddressSettings): luna-python-matcher address settings
        additionalServicesUsage(AdditionalServicesUsageSetting): support services container
        remoteSDKAddress(_ServiceAddressSettings): external luna remote sdk address
        remoteSDKTimeouts(ServiceTimeoutsSettings): luna-remote-sdk timeouts settings
        videoAgentAddress(_ServiceAddressSettings): external luna video agent address
        videoAgentTimeouts(ServiceTimeoutsSettings): luna-video-agent timeouts settings
        senderAddress (ServiceAddressSettings): luna-sender address settings
    """

    serviceName = "luna-lambda-unit"
    settingsStore = SETTINGS

    def __init__(self):
        super().__init__()
        self.faceSamplesStorage: _ImageStoreAddressSettings | None = None
        self.bodySamplesStorage: _ImageStoreAddressSettings | None = None
        self.imageOriginStorage: _ImageStoreAddressSettings | None = None
        self.faceSamplesStorageTimeouts: _ServiceTimeoutsSettings | None = None
        self.bodySamplesStorageTimeouts: _ServiceTimeoutsSettings | None = None
        self.imageOriginStorageTimeouts: _ServiceTimeoutsSettings | None = None
        self.facesAddress: _ServiceAddressSettings | None = None
        self.facesTimeouts: _ServiceTimeoutsSettings | None = None
        self.eventsAddress: _ServiceAddressSettings | None = None
        self.eventsTimeouts: _ServiceTimeoutsSettings | None = None
        self.pythonMatcherAddress: _ServiceAddressSettings | None = None
        self.pythonMatcherTimeouts: _ServiceTimeoutsSettings | None = None
        self.matcherProxyAddress: _ServiceAddressSettings | None = None
        self.matcherProxyTimeouts: _ServiceTimeoutsSettings | None = None
        self.additionalServicesUsage: _AdditionalServicesUsageSetting | None = None
        self.remoteSDKAddress: _ServiceAddressSettings | None = None
        self.remoteSDKTimeouts: _ServiceTimeoutsSettings | None = None
        self.videoAgentAddress: _ServiceAddressSettings | None = None
        self.videoAgentTimeouts: _ServiceTimeoutsSettings | None = None
        self.senderAddress: _ServiceAddressSettings | None = None

    def applySettings(self):
        """
        Deserialize loaded settings to classes
        """
        super().applySettings()
        self.faceSamplesStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_FACES_SAMPLES_ADDRESS",
            self["LUNA_IMAGE_STORE_FACES_SAMPLES_ADDRESS"],
        )
        self.bodySamplesStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_BODIES_SAMPLES_ADDRESS",
            self["LUNA_IMAGE_STORE_BODIES_SAMPLES_ADDRESS"],
        )
        self.imageOriginStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_IMAGES_ADDRESS",
            self["LUNA_IMAGE_STORE_IMAGES_ADDRESS"],
        )
        self.faceSamplesStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_FACES_SAMPLES_TIMEOUTS",
            self["LUNA_IMAGE_STORE_FACES_SAMPLES_TIMEOUTS"],
        )
        self.bodySamplesStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_BODIES_SAMPLES_TIMEOUTS",
            self["LUNA_IMAGE_STORE_BODIES_SAMPLES_TIMEOUTS"],
        )
        self.imageOriginStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_IMAGES_TIMEOUTS",
            self["LUNA_IMAGE_STORE_IMAGES_TIMEOUTS"],
        )

        self.facesAddress = _ServiceAddressSettings(self.serviceName, "LUNA_FACES_ADDRESS", self["LUNA_FACES_ADDRESS"])
        self.facesTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_FACES_TIMEOUTS", self["LUNA_FACES_TIMEOUTS"]
        )

        self.eventsAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_EVENTS_ADDRESS",
            self["LUNA_EVENTS_ADDRESS"],
        )
        self.eventsTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_EVENTS_TIMEOUTS",
            self["LUNA_EVENTS_TIMEOUTS"],
        )

        self.pythonMatcherAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_PYTHON_MATCHER_ADDRESS",
            self["LUNA_PYTHON_MATCHER_ADDRESS"],
        )
        self.pythonMatcherTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_PYTHON_MATCHER_TIMEOUTS",
            self["LUNA_PYTHON_MATCHER_TIMEOUTS"],
        )

        self.matcherProxyAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_MATCHER_PROXY_ADDRESS", self["LUNA_MATCHER_PROXY_ADDRESS"]
        )
        self.matcherProxyTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_PYTHON_MATCHER_PROXY_TIMEOUTS", self["LUNA_PYTHON_MATCHER_PROXY_TIMEOUTS"]
        )

        self.additionalServicesUsage = _AdditionalServicesUsageSetting(
            self.serviceName,
            self["ADDITIONAL_SERVICES_USAGE"],
        )

        self.remoteSDKAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_REMOTE_SDK_ADDRESS",
            self["LUNA_REMOTE_SDK_ADDRESS"],
        )
        self.remoteSDKTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_REMOTE_SDK_TIMEOUTS",
            self["LUNA_REMOTE_SDK_TIMEOUTS"],
        )
        self.videoAgentAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_VIDEO_AGENT_ADDRESS",
            self["LUNA_VIDEO_AGENT_ADDRESS"],
        )
        self.videoAgentTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_VIDEO_AGENT_TIMEOUTS",
            self["LUNA_VIDEO_AGENT_TIMEOUTS"],
        )

        self.senderAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_SENDER_ADDRESS", self["LUNA_SENDER_ADDRESS"]
        )

        return self
